% Jialiang Wang
% June 23, 2014
% Modified for tutorial demo on Nov 12, 2014
%
% You need to change the directories below to your image directories,
% DPM detector directory, and libsvm directory prior to usage
% Also, prior to usage,
% compile DPM helper code and libsvm matlab API
% See DPM and libsvm's README file for detail

function classifier_car_vs_motorcycle()

%--------------------------
% Part1: Train SVM Classifier
%--------------------------

    % Change to your path and your file extensions
    TRAINING_POS = fullfile(pwd, 'train_images', 'cars');
    TRAINING_NEG = fullfile(pwd, 'train_images', 'motorcycles');
    TESTING_POS = fullfile(pwd, 'test_images', 'cars');
    TESTING_NEG = fullfile(pwd, 'test_images', 'motorcycles');

    TrainingImgs = dir([TRAINING_POS '/' '*.png']);
    TrainingNegs = dir([TRAINING_NEG '/' '*.jpeg']);
    TestingImgs = dir([TESTING_POS '/' '*.png']);
    TestingNegs = dir([TESTING_NEG '/' '*.jpeg']);

    NumTrainImgs = size(TrainingImgs,1); 
    NumTrainNegs = size(TrainingNegs, 1);
    NumTestImgs = size(TestingImgs, 1);
    NumTestNegs = size(TestingNegs, 1);

    % Add your own DPM detector directory
    addpath(fullfile(pwd, 'LSVM_sup_original'));
    addpath(fullfile(pwd, 'libsvm', 'matlab'));   
    % First, calculate the average height and width
    disp('-------------------------------------------');
    disp('Reading images');
    disp('-------------------------------------------');

    TotalWidth = 0;
    TotalHeight = 0;
    count = 0;
    
    % Here I assumed I have equal amount of train pos, train neg,
    % test pos, test neg. Otherwise, we just need to change a little 
    % bit of the code to get the average
    for i=1:NumTrainImgs,
         Trainingnegimage = imread([TRAINING_NEG '/' TrainingNegs(i).name]);
         Trainingimage = imread([TRAINING_POS '/' TrainingImgs(i).name]);
         TotalWidth = TotalWidth + length(Trainingimage(:,1,1)) + length(Trainingnegimage(:,1,1));
         TotalHeight = TotalHeight + length(Trainingimage(1,:,1)) + length(Trainingnegimage(1,:,1));
         count = count + 1;
     end

     AverageHeight = TotalHeight/(NumTrainNegs+ NumTrainImgs)
     AverageWidth = TotalWidth/(NumTrainNegs+ NumTrainImgs)


    disp('-------------------------------------------');
    disp('Warping training images and extracting features');
    disp('-------------------------------------------');
    disp('Positives');
    disp('-------------------------------------------');

    training_labels = [];
    training_instances = [];
    testing_labels = [];
    testing_instances = [];
    percentage_counter = 0;
    m = 1;
    n = 1;

    for i = 1:NumTrainImgs
        training_labels(m) = 1;
        image = imread([TRAINING_POS '/' TrainingImgs(i).name]);
        image = imresize(image, [AverageWidth, AverageHeight]);
        image = color(image);
        image = im2double(image);
        f = features(image, 8);
        
        % for demo purpose only
        % visualize the image and its HOG featues
        if i==1
            clf;
            imshow(image);
            disp('first positive training image -- car');
            disp('press any key to visualize its HOG feature'); pause;
            imHOG = visualizeHOG(f);
            disp('press any key to continue');pause;
        end
        % end of demo part
        
        count = 1;
        for a = 1:length(f(:,1,1))
            for b = 1:length(f(1,:,1))
                for c = 1:length(f(1,1,:))
                    training_instances(count,m) = f(a,b,c);
                    count = count+1;
                end
            end
        end
        m = m+1;
       
        percentage_counter = percentage_counter + 1;
        if percentage_counter >= 20
            disp(sprintf('%2f %% finished', 100*i/NumTrainImgs));
            percentage_counter = 0;
        end
    end
    NumOfPos = m - 1;
    
    disp('-------------------------------------------');
    disp('----------   Negatives    -----------------');
    disp('-------------------------------------------');
    for i = 1:NumTrainNegs
        training_labels(m) = -1;
        image = imread([TRAINING_NEG '/' TrainingNegs(i).name]);
        image = imresize(image, [AverageWidth, AverageHeight]);
        image = color(image);
        image = im2double(image);
        f = features(image, 8);

        % for demo purpose only
        % visualize the image and its HOG featues
        if i==5
            clf;
            imshow(image);
            disp('first negative training image -- motorcycle');
            disp('press any key to visualize its HOG feature'); pause;
            imHOG = visualizeHOG(f);
            disp('press any key to continue');pause;
        end
        % end of demo part        
        
        count = 1;
        for a = 1:length(f(:,1,1))
            for b = 1:length(f(1,:,1))
                for c = 1:length(f(1,1,:))
                    training_instances(count,m) = f(a,b,c);
                    count = count+1;
                end
            end
        end
        m = m+1;
       
        percentage_counter = percentage_counter + 1;
        if percentage_counter >= 20
            disp(sprintf('%2f %% finished', 100*i/NumTrainNegs));
            percentage_counter = 0;
        end
    end

    training_labels = sparse(training_labels');
    training_instances = sparse(training_instances');
    training_instances = scale_set(training_instances);

    NumOfNeg = m - 1 - NumOfPos;
    
    disp('-------------------------------------------');
    disp('Start SVM Training');
    disp('-------------------------------------------');

    % set parameter:
    % I only balanced the pos and neg's weight
    % There are more parameters
    % You could try to see which combination gives you the best performance
    
    parameter = ['-w1 ', ' ', num2str(NumOfNeg), ' -w-1 ', ' ', num2str(NumOfPos)];
    model = svmtrain(training_labels, training_instances, parameter);
    disp('Congrats! The model has successully built!');
    disp('Now we can detect cars vs motorcycles');
    disp('press any key to continue'); pause;
    disp('demo detection starts');

    
    %--------------------------------------
    % Part 2: Detection of a single image
    %---------------------------------------
    
    disp('-------------------------------------------');
    disp('Now let us test a car -- a positive example');
    disp('-------------------------------------------');
    disp('press any key to continue'); pause;
    
    testing_labels(1) = 1;
    image = imread([TESTING_POS '/' TestingImgs(3).name]);
    image = imresize(image, [AverageWidth, AverageHeight]);
    image = color(image);
    image = im2double(image);
    f = features(image, 8);
    imshow(image);

    count = 1;
    for a = 1:length(f(:,1,1))
        for b = 1:length(f(1,:,1))
            for c = 1:length(f(1,1,:))
                testing_instances(count,1) = f(a,b,c);
                count = count+1;
            end
        end
    end
    
    testing_labels = sparse(testing_labels');
    testing_instances = sparse(testing_instances');
    testing_instances = scale_set(testing_instances);
    

    [predicted_labels, accuracy, dec_values] = svmpredict(testing_labels, testing_instances, model);
    disp('----')
    disp('As Sanja said in class, a good positive prediction has a decision value of >1')
    disp('whereas a good negative prediction has a decision value of < -1')
    disp('Our decision value is:');
    disp(dec_values);
    disp('car detected, woohoo!'); pause;
    
    disp('-------------------------------------------');
    disp('Now let us test a motorcycle -- a negative example');
    disp('-------------------------------------------');
    disp('press any key to continue'); pause;
    
    testing_labels = [];
    testing_instances = [];
    n=1;
    testing_labels(1) = -1;
    image = imread([TESTING_NEG '/' TestingNegs(3).name]);
    image = imresize(image, [AverageWidth, AverageHeight]);
    image = color(image);
    image = im2double(image);
    f = features(image, 8);
    imshow(image);

    count = 1;
    for a = 1:length(f(:,1,1))
        for b = 1:length(f(1,:,1))
            for c = 1:length(f(1,1,:))
                testing_instances(count,1) = f(a,b,c);
                count = count+1;
            end
        end
    end
    
    testing_labels = sparse(testing_labels');
    testing_instances = sparse(testing_instances');
    testing_instances = scale_set(testing_instances);

    [predicted_labels, accuracy, dec_values] = svmpredict(testing_labels, testing_instances, model);
    
    disp('motorcycle detected!');
    disp('decision value is:');
    disp(dec_values);
    disp('press any key to continue'); pause;

    %-------------------------------
    % Part 3: Detection of more images
    %---------------------------------
    
    testing_labels = [];
    testing_instances = [];
    disp('-------------------------------------------');
    disp('Now let us try to detect more images');
    disp('Half of them are cars and half of them are motorcycles');
    disp('press any key to continue'); pause;
    disp('-------------------------------------------');
    percentage_counter = 0;

    disp('-------------------------------------------');
    disp('Warping training images and extracting features');
    disp('-------------------------------------------');
    n = 1;
    for i = 1:NumTestImgs
        testing_labels(n) = 1;
        image = imread([TESTING_POS '/' TestingImgs(i).name]);
        image = imresize(image, [AverageWidth, AverageHeight]);
        image = color(image);
        image = im2double(image);
        f = features(image, 8);
        
        count = 1;
        for a = 1:length(f(:,1,1))
            for b = 1:length(f(1,:,1))
                for c = 1:length(f(1,1,:))
                    testing_instances(count,n) = f(a,b,c);
                    count = count+1;
                end
            end
        end
        n = n+1;
       
        percentage_counter = percentage_counter + 1;
        if percentage_counter >= 20
            disp(sprintf('%2f %% finished', 100*i/NumTestImgs));
            percentage_counter = 0;
        end
    end

    disp('------ negative testing images  --------')
    for i = 1:NumTestNegs
        testing_labels(n) = -1;
        image = imread([TESTING_NEG '/' TestingNegs(i).name]);
        image = imresize(image, [AverageWidth, AverageHeight]);
        image = color(image);
        image = im2double(image);
        f = features(image, 8);

        count = 1;
        for a = 1:length(f(:,1,1))
            for b = 1:length(f(1,:,1))
                for c = 1:length(f(1,1,:))
                    testing_instances(count,n) = f(a,b,c);
                    count = count+1;
                end
            end
        end
        n = n+1;
       
        percentage_counter = percentage_counter + 1;
        if percentage_counter >= 20
            disp(sprintf('%2f %% finished', 100*i/NumTrainNegs));
            percentage_counter = 0;
        end
    end

    testing_labels = sparse(testing_labels');
    testing_instances = sparse(testing_instances');
    testing_instances = scale_set(testing_instances);

    [predicted_labels, accuracy, dec_values] = svmpredict(testing_labels, testing_instances, model);
end

function scaledI = scale_set(I)
    scaledI = I ./ (max(I(:)-min(I(:))));
end